@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2023 Artifex Software, Inc.
+// Copyright (C) 2004-2024 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -76,7 +76,7 @@ struct pdf_processor
 	void (*op_gs_BM)(fz_context *ctx, pdf_processor *proc, const char *blendmode);
 	void (*op_gs_ca)(fz_context *ctx, pdf_processor *proc, float alpha);
 	void (*op_gs_CA)(fz_context *ctx, pdf_processor *proc, float alpha);
-	void (*op_gs_SMask)(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, float *bc, int luminosity, pdf_obj *tr);
+	void (*op_gs_SMask)(fz_context *ctx, pdf_processor *proc, pdf_obj *smask, fz_colorspace *smask_cs, float *bc, int luminosity, pdf_obj *tr);
 	void (*op_gs_end)(fz_context *ctx, pdf_processor *proc);
 
 	/* special graphics state */
@@ -324,11 +324,15 @@ struct pdf_filter_options
 
 typedef enum
 {
-	FZ_CULL_PATH_FILL,
-	FZ_CULL_PATH_STROKE,
-	FZ_CULL_PATH_FILL_STROKE,
-	FZ_CULL_CLIP_PATH,
-	FZ_CULL_GLYPH,
+	FZ_CULL_PATH_DROP = 0,
+	FZ_CULL_PATH_FILL = 1,
+	FZ_CULL_PATH_STROKE = 2,
+	FZ_CULL_PATH_FILL_STROKE = 3,
+	FZ_CULL_CLIP_PATH_DROP = 4,
+	FZ_CULL_CLIP_PATH_FILL = 5,
+	FZ_CULL_CLIP_PATH_STROKE = 6,
+	FZ_CULL_CLIP_PATH_FILL_STROKE = 7,
+	FZ_CULL_GLYPH = 8,
 	FZ_CULL_IMAGE,
 	FZ_CULL_SHADING
 } fz_cull_type;
@@ -483,7 +487,7 @@ void pdf_tos_save(fz_context *ctx, pdf_text_object_state *tos, fz_matrix save[2]
 void pdf_tos_restore(fz_context *ctx, pdf_text_object_state *tos, fz_matrix save[2]);
 fz_text *pdf_tos_get_text(fz_context *ctx, pdf_text_object_state *tos);
 void pdf_tos_reset(fz_context *ctx, pdf_text_object_state *tos, int render);
-int pdf_tos_make_trm(fz_context *ctx, pdf_text_object_state *tos, pdf_text_state *text, pdf_font_desc *fontdesc, int cid, fz_matrix *trm);
+int pdf_tos_make_trm(fz_context *ctx, pdf_text_object_state *tos, pdf_text_state *text, pdf_font_desc *fontdesc, int cid, fz_matrix *trm, float *adv);
 void pdf_tos_move_after_char(fz_context *ctx, pdf_text_object_state *tos);
 void pdf_tos_translate(pdf_text_object_state *tos, float tx, float ty);
 void pdf_tos_set_matrix(pdf_text_object_state *tos, float a, float b, float c, float d, float e, float f);
